/*
 * Created by Stefan Korecko, 2016-18
 */


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Poznámky

/*
Ak by som chcel overiť validitu samostatne, zavolám na formulár alebo konkrétny prvok
checkValidity()
POZOR: toto síce overí validitu, ale bez klasických html výpisov
Viac o js funkcionalite pre HTML5 validáciu:

http://www.w3schools.com/js/js_validation_api.asp
*/


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Kód, ktorý sa vykoná pri načítaní skriptu
// Code executed when the script is loaded

var $form = $("#SKorFrm");
var artId = queryString2obj().id;


if (isFinite(artId)){
    console.log("Edit article "+artId);

    $.ajax({
        type: 'GET',
        url: "http://"+server+"/api/article/"+artId,
        dataType: "json",
        success: function (article) {
            $("#author").val(article.author);
            $("#title").val(article.title);
            $("#imageLink").val(article.imageLink);
            $("#content").val(article.content);
            $("#tags").val(article.tags);
            $("#frmTitle").html("Uprav článok / Edit article");
        },
        error:function(jxhr){
            errorAlert("Načitanie článku zlyhalo (article loading failed).",jxhr);
        }
    });
}else{
    $("#frmTitle").html("Pridaj článok / Add article");
}



//Pridanie funkcionality pre kliknutie na tlacidlo "Späť / Back"
//Adding functionality for the button "Späť / Back"
$("#btBack").click(function(){
    window.history.back()
});

//Pridanie funkcionality pre kliknutie na tlacidlo "Ulož článok / Save article"
//Adding functionality for the button "Ulož článok / Save article"
$form.submit(function(event){  //tu potrebujem aj objekt s udalosťou, aby som (here I also need the event object)
    event.preventDefault();  //zrušil pôvodné spracovanie udalosti (to cancel the default event handling)
    if (isFinite(artId)) {
        prepareAndSendArticle($form,"PUT","http://"+server+"/api/article/"+artId);
    }else{
        prepareAndSendArticle($form,"POST","http://"+server+"/api/article");
    }

});

//Pridanie funkcionality pre kliknutie na tlacidlo "Nahraj obrázok / Upload image"
//Adding functionality for the button "Nahraj obrázok / Upload image"
$("#btShowFileUpload").click(function(){
    $('#fsetFileUpload').removeClass("skryty");
    $('#btShowFileUpload').addClass("skryty");

});


//Pridanie funkcionality pre kliknutie na tlacidlo "Odošli obrázok na server / Send image to server"
//Adding functionality for the button "Odošli obrázok na server / Send image to server"
$("#btFileUpload").click(function(){
    uploadImg(
        $('#imageLink'),
        $('#fsetFileUpload'),
        $('#btShowFileUpload'),
        document.getElementById('flElm').files //vanilla way, jQuery version doesn't have files
    );
});

//Pridanie funkcionality pre kliknutie na tlacidlo "Zruš nahrávanie / Cancel uploading"
//Adding functionality for the button "Zruš nahrávanie / Cancel uploading"
$("#btCancelFileUpload").click(function(){
    $('#fsetFileUpload').addClass("skryty");
    $('#btShowFileUpload').removeClass("skryty");
});
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//funkcie
//functions

/**
 * Spracuje údaje o článku z formulára a odošle na uloženie na server
 * @param $frm - formulár s článkom (jQuery objekt)
 * @param method - metóda, "POST" (pridanie článku) alebo "PUT" (úprava článku)
 * @param restURL - url zdroja na serveri
 *
 * Processes article data from the form and sends them to the server.
 * @param $frm - article form (jQuery object)
 * @param method - method, "POST" (add article) or "PUT" (edit article)
 * @param restURL - url of the resource at the server
 */
function prepareAndSendArticle($frm, method, restURL) {


    //1. Uloží údaje z formulára do objektu data
    //1. Puts form data to the object data
    var data = {};
    $frm.serializeArray().map(
        function(item){
            var itemValueTrimmed = item.value.trim();
            if(itemValueTrimmed){//if the value is non-empty string
                data[item.name] = item.value;
            }
        }
    );


    console.log("prepareAndSendArticle> Data from the form in the data object:");
    console.log(JSON.stringify(data));

    //2.Upraví objekt data do podoby vhodnej na odoslanie
    //2.Modifies the data object to a form suitable for sending
    if(data.tags){ //if there is the item tags and it it not an empty string.
                   //If I just need to know whether there is the item tags, it is better to call
                   //Object.prototype.hasOwnProperty.call(data, 'tags')
                   //see https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object/hasOwnProperty
        data.tags=data.tags.split(","); //zmeni retazec na pole. Oddelovac poloziek je ciarka.
        data.tags=data.tags.map(function(tag) {return tag.trim()}); //odstráni prázdne znaky na začiatku a konci každého kľúčového slova
    }

    console.log("prepareAndSendArticle> Form data successfully converted to:");
    console.log(JSON.stringify(data));

    //3.Kontrola, či boli zadané povinné polia
    //3.Required/format validation
    if(!data.title){ //this is just for sure
        alert("Názov článku musí byť zadaný a musí obsahovať čitateľné znaky\nArticle title has to be entered and has to contain readable characters only.");
        return;
    }
    if(!data.content){ //this is important, checks whether the user entered only white space characters.
        alert("Obsah článku musí byť zadaný a musí obsahovať čitateľné znaky.\nArticle content has to be entered and has to contain readable characters only");
        return;
    }

    console.log("prepareAndSendArticle> Form data validated.");


    //4. odoslanie údajov
    //4. sending the data
    if(window.confirm("Skutočne si želáte článok zapísať do databázy?\nDo you really wish to upload the article?")){

        $.ajax({
            type: method,
            url: restURL,
            dataType: "json",
            contentType:"application/json;charset=UTF-8",
            data:JSON.stringify(data),
            success: function (response) {
                if(response.id){
                    console.log(response.id);
                    window.location.href="article.html?id="+response.id;
                }
            },
            error: function (jxhr) {
                errorAlert("Zapísanie článku zlyhalo (article uploading failed).",jxhr);
            }
        });

    }
}


/**
 * Uploads an image to the server
 * @param $imgLinkElement - css selector for the id of the input type="url" element, where the link of the uploaded file will be added arter the upload
 * @param $fieldsetElement - css selector for the id of the hideable fieldset element, which conains the controls for the file upload.
 * @param $btShowFileUploadElement - css selector for the id of the button type="button" element, which shows or hides the fieldset
 * @param files - a FileList object with the image to be uploaded as the first item.
 */
function uploadImg($imgLinkElement,$fieldsetElement, $btShowFileUploadElement, files) {
    if (files.length>0){
        var imgData = new FormData();
        imgData.append("file", files[0]); //beriem len prvy obrazok, ved prvok formulara by mal povolit len jeden
                                          //takes only the first file (image)
        //pozor:nezadavat content-type. potom to nepojde.
        //Beware: It doesn't work correctly if the content-type is set.
        $.ajax({
            type: "POST",
            url: "http://"+server+"/api/fileUpload",
            dataType: "json",
            processData: false, //toto musim dat, aby sa jQuery nesnazil udaje spracovavat. inak vyvola  “Uncaught TypeError: Illegal invocation”
                                //this is here to prevent data processing by jQuery. Otherwise the  “Uncaught TypeError: Illegal invocation” exception is invoked.
            contentType: false, //toto musim dat, aby server poziadavku spracoval
                                //this has to be here for the server to process the request correctly
            data:imgData,
            success: function (response) {
                if(response.fullFileUrl){
                    $imgLinkElement.val(response.fullFileUrl);
                    $btShowFileUploadElement.removeClass("skryty");
                    $fieldsetElement.addClass("skryty");
                }
            },
            error: function (jxhr) {
                errorAlert("pracovanie neúspešné. Obrázok nebol uložený na serveri (image uploading failed).",jxhr);
            }
        });
    }else{
        window.alert("Vyberte súbor s obrázkom.\nPlease, choose an image file.");
    }



}
