/*
 * Created by Stefan Korecko, 2016-18
 */

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Kód, ktorý sa vykoná pri načítaní skriptu
//Code executed when the script is loaded

console.log("Zacinam stahovat zoznam clankov / Starting articles download ...");


//Výpis prvých maximálne articlesPerPage článkov
//Write first articlesPerPage articles to html
writeArticles2Html(0, articlesPerPage, server, 'clanky');



////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//funkcie
//functions


/**
 * prida k objektu articles objekt s udajmi pre navigacne tlacidla
 * @param articles - objekt s clankami, ku ktoremu sa pridaju udaje pre navigacne tlacidla.
 * @param startIndex - index (poradové číslo článku od 0) od ktorého sa články vypisujú
 * @returns {Object} - objekt articles  s pridanymi udajmi pre navigacne tlacidla
 *
 * Adds an object with navigation buttons data to the object articles
 * @param articles - object with articles.
 * @param startIndex - sequence number of the first of the displayed articles
 * @returns {Object} - the object with articles with the navigation buttons data
 */

function addNavBtInfo(articles, startIndex){
    if(startIndex>0) {
        articles.prev = {
            from:  (startIndex - articlesPerPage > 0 ? startIndex - articlesPerPage : 0),
        };
    }
    if(startIndex+articlesPerPage<articles.meta.totalCount){
        articles.nxt={
            from:  startIndex+articlesPerPage,
        };
    }
}


/**
 * Zapíše údaje o článkoch do elementu s id articlesElmId.
 * HTML kód pre navigáciu negeneroje (je potrebné doplniť)
 * Mustache sablona sa nacitava so samostatneho subora templates/listOfArticles.mst
 * @param startIndex - index (poradové číslo čláanku od 0) od ktorého sa články vypisujú
 * @param max - maximálny počet článkov.
 * @param server - doménové meno servera odkiaľ sa majú údaje stiahnuť.
 * @param articlesElmId - Id elementu do ktorého sa články majú vypísať
 *
 * Writes articles data to the HTML element with id=articlesElmId
 * Does not generete an HTML code of the navigation (needs to be added)
 * The mustache template is from a separate file, templates/listOfArticles.mst
 * @param startIndex - the article sequence number (from 0) from which articles are written
 * @param max - maximum number of articles.
 * @param server - domain name of the server with the article database
 * @param articlesElmId - Id of the element to which the articles are to be added
 */
function writeArticles2Html(startIndex, max, server, articlesElmId){
    $.ajax({
        type: 'GET',
        url: "http://"+server+"/api/article/",
        data: { max: max, offset: startIndex },
        dataType: "json",
        success: function (articles) {
            addNavBtInfo(articles,startIndex);
            $.get("templates/listOfArticles.mst",      //get() is a special version of ajax()
                function (template) {
                    $("#"+articlesElmId).html(Mustache.render(template, articles));
                }
                ,"text");
        },
        error:function(jxhr){
            errorAlert("Načitanie článkov zlyhalo.\nArticles download failed.",jxhr);
        }
    });
}